/**
 * @file
 * Drupal Inspect module
 */

/**
 * @param {func} jQuery
 */
(function($) {
  /**
   * @ignore
   */
  Drupal.inspect = window.Inspect = window.inspect = (function() {
    var _cls = "module-inspect-collapsible", _shw = 1, _fld = 1,
    _opts = ["depth", "protos", "func_body", "message", "category", "severity"], _nOpts = _opts.length,
    _strQt = [">>", "<<"],
    _loc,
    _dmnRgx = new RegExp( (_loc = window.location.href).replace(/^(https?:\/\/[^\/]+)\/.*$/, "$1").replace(/([\/\.\-])/g,"\\"+"$1") ),
    _logNo = 0,
    _oKeys, _rslv, _trc, _typeOf, _cnsl, _nspct, _fL, _log, _lgM, _ua,
    f;
    /**
     * Object.keys() for IE<8.
     *
     * @ignore
     * @private
     * @param {object} o
     * @return {arr}
     */
    _oKeys = Object.keys ? function(o) {
      return Object.keys(o);
    } : function(o) {
      var k, a = [];
      for(k in o) {
        if(o.hasOwnProperty(k)) {
          a.push(k);
        }
      }
      return a;
    };
    /**
     * Resolve options argument.
     *
     * @ignore
     * @private
     * @param {mixed} u
     * @return {obj}
     */
    _rslv = function(u) {
      var o = {}, i, t;
      if(!u || (t = typeof u) !== "object") {
        for(i = 0; i < _nOpts; i++) {
          o[ _opts[i] ] = undefined;
        }
        if(u) {
          switch(t) {
            case "number":
              if(isFinite(u)) {
                o.depth = u;
              }
              break;
            case "string":
              switch(u) {
                case "protos":
                  o.protos = true;
                  break;
                case "func_body":
                  o.func_body = true;
                  break;
                default:
                  o.message = u;
              }
              break;
            case "boolean":
              if(u) {
                o.protos = true;
              }
              break;
          }
        }
        else if(u === 0) {
          o.depth = 0;
        }
      }
      else {
        for(i = 0; i < _nOpts; i++) {
          o[ _opts[i] ] = u.hasOwnProperty(_opts[i]) ? u[ _opts[i] ] : undefined;
        }
      }
      return o;
    };
    /**
     * Send message to browser console, if exists.
     *
     * @ignore
     * @private
     * @param {mixed} [ms]
     * @return {void}
     */
    _cnsl = function(ms) {
      try { // detecting console is not possible, thus try-catch
        console.log("" + ms);
      }
      catch(er) {}
    };
    /**
     * Trace and stringify error.
     *
     * @ignore
     * @private
     * @param {Error} er
     * @return {str}
     */
    _trc = function(er) {
      var u, le, i, es = ""+er, s = es;
      if((u = er.stack)) { // gecko, chromium
        if((le = (u = (u.replace(/\r/, "").split(/\n/))).length)) {
          i = u[0] === es ? 1 : 0; // chromium first line is error toString()
          for(i; i < le; i++) {
            s += "\n  " + u[i].replace(/^[\ ]+at\ /, "@"). // chromium as gecko
                replace(_dmnRgx, "");
          }
        }
      }
      return s;
    };
    /**
     * All native types are reported in lowercase (like native typeof does).
     * If given no arguments: returns "inspect".
     * Types are:
     * - native, typeof: object string number
     * - native, corrected: function array date regexp image
     * - window, document, document.documentElement (not lowercase)
     * - element, checked via .getAttributeNode
     * - text node: textNode
     * - attribute node: attributeNode
     * - event: event (native and prototyped W3C Event and native IE event)
     * - jquery
     * - emptyish and bad: undefined, null, NaN, infinite
     * - custom or prototyped native: all classes having a typeOf() method.
     * RegExp is an object of type regexp (not a function - gecko/webkit/chromium).
     * Does not check if Date object is NaN.
     *
     * @ignore
     * @private
     * @param {mixed} u
     * @return {str}
     */
    _typeOf = function(u) {
      var t = typeof u;
      if(!arguments.length) {
        return "inspect";
      }
      switch(t) {
        case "boolean":
        case "string":
            return t;
        case "number":
          return isFinite(u) ? t : (isNaN(u) ? "NaN" : "infinite");
        case "object":
          if(u === null) {
            return "null";
          }
          //  Accessing properties of object may err for various reasons, like missing permission (Gecko).
          try {
            if(u.typeOf && typeof u.typeOf === "function") {
              return u.typeOf();
            }
            else if(typeof u.length === "number" && !(u.propertyIsEnumerable("length")) && typeof u.splice === "function") {
              return "array";
            }
            else if(u === window) {
              return "window";
            }
            else if(u === document) {
              return "document";
            }
            else if(u === document.documentElement) {
              return "document.documentElement";
            }
            else if(u.getAttributeNode) { // element
              //  document has getElementsByTagName, but not getAttributeNode -  document.documentElement has both
              return u.tagName.toLowerCase === "img" ? "image" : "element";
            }
            else if(u.nodeType) {
              switch(u.nodeType) {
                case 3:return "textNode";
                case 2:return "attributeNode";
              }
              return "otherNode";
            }
            else if(typeof u.stopPropagation === "function" ||
                (u.cancelBubble !== undefined && typeof u.cancelBubble !== "function" &&
                typeof u.boundElements === "object")) {
              return "event";
            }
            else if(typeof u.getUTCMilliseconds === "function") {
              return "date";
            }
            else if(typeof u.exec === "function" && typeof u.test === "function") {
              return "regexp";
            }
            else if(u.hspace && typeof u.hspace !== "function") {
              return "image";
            }
            else if(u.jquery && typeof u.jquery === "string" && !u.hasOwnProperty("jquery")) {
              return "jquery";
            }
          }
          catch(er) {
          }
          return t;
        case "function":
          //  gecko and webkit reports RegExp as function instead of object
          return (u.constructor === RegExp || (typeof u.exec === "function" && typeof u.test === "function")) ?
              "regexp" : t;
      }
      return t;
    };
    /**
     * Analyze variable.
     *
     * @ignore
     * @private
     * @param {mixed} u
     * @param {bool} [protos]
     *  - default: false
     *  - include non-native prototypals
     * @param {bool} [func_body]
     *  - default: false
     *  - print bodies of functions
     * @param {int} [max]
     *  - default: 10
     *  - depth limiter
     * @param {int} [depth]
     *  - default: zero
     *  - current depth
     * @return {str}
     */
    _nspct = function(u, protos, func_body, max, depth) {
      var m = max !== undefined ? max : 10, d = depth || 0, fb = func_body,
        t, isArr, p, pT, v, buf = "", s, isProt, nInst = 0, nProt = 0, ind, fInd, i, em;
      if(m > 10 || m < 0) {
        m = 10;
      }
      if(d < 11) {
        if(u === undefined) {
          return "(undefined)\n";
        }
        //  resolve func_body ------------------------
        if(!d && fb === undefined && (v = Drupal.settings.inspect)) {
          fb = v.func_body;
        }
        //  check type by typeof ---------------------
        switch((t = typeof u)) {
          case "object":
            if(u === null) {
              return "(object) null\n";
            }
            break;
          case "boolean":
            return "(" + t + ") " + u.toString() + "\n";
          case "string":
            return "(" + t + ":" + u.length + ") " + _strQt[0] +
              u.replace(/\n/g, "_NL_").replace(/\r/g, "_CR_").replace(/\t/g, "_TB_") +
              _strQt[1] + "\n";
        }
        //  check by typeOf() ------------------------
        ind = new Array(d + 2).join(".  ");
        switch((t = _typeOf(u))) {
          case "number":
            return "(" + t + ") " + u + "\n";
          case "NaN":
            return "(NaN) NaN\n";
          case "infinite":
            return "(" + t + ") infinite\n";
          case "regexp":
            return "(" + t + ") " + u.toString() + "\n";
          case "date":
            return "(date) " + (u.toISOString ? u.toISOString() : u) + "\n";
          case "function":
            //  find static members of the function
            try {
              for(p in u) {
                if(u.hasOwnProperty(p) && p !== "prototype") {
                  ++nInst;
                  buf += ind + p + ": " + _nspct(u[p], protos, fb, m - 1, d + 1); // recursion
                }
              }
            }
            catch(er1) {}
            v = u.toString();
            if(fb && (i = (v = v.replace(/\r/g, "").replace(/\n$/, "")).search(/[\ ]+\}$/) ) > -1) { // remove indentation of function as a whole
              v = v.replace( new RegExp("\\n" + new Array(v.length - i).join("\\ "), "g"), "\n");
            }
            return "(" + t + ":" + nInst + ") {\n" +
                (fInd = new Array(d + 2).join("   ")) +
                (!fb ? v.substr(0, v.indexOf(")") + 1) :
                    v.replace(/\n/g, "\n" + fInd)
                ) +
                "\n" +
                buf + ind.replace(/\.\ \ $/, "") + "}\n";
          case "window":  case "document":  case "document.documentElement":
            return "(" + t + ")\n";
          case "element":
            return "(" + t + ":" + (u.tagName ? u.tagName.toLowerCase() : "") +  // events may have elements, with no tagName
                ") " + (u.id ? u.id : "-") + "|" + (u.className ? u.className : "-") +
                (!(u = u.getAttribute("name")) ? "" : ("|"+u)) + "\n";
          case "textNode":  case "attributeNode":  case "otherNode":
            s = "(" + t + (t === "otherNode" ? ":" + u.nodeType + ") " : " ");
            try { // dumping nodeValue in IE<8 will fail
              return s + _nspct(u.nodeValue, protos, fb, m - 1, d + 1); // recursion
            } catch(er2) {}
            return s + "Unknown node value (IE)\n";
          case "image":
            return "(image) " + u.src + "\n";
          case "event":
            break;
        }
        //  object -------------------------------------------------------
        isArr = (t === "array");
        buf = ["", "", ""]; // instance properties, prototypal attributes, prototypal methods
        em = "";
        try {
          pT = "not iterable";
          for(p in u) {
            pT = _typeOf(v = u[p]);
            //  event misses hasOwnProperty method in some browsers, so we have to check if hasOwnProperty exists
            if(u.hasOwnProperty && u.hasOwnProperty(p)) {
              if(isArr) { // do always check for non-numeric instance properties if array (error)
                if((""+p).search(/\D/) > -1) {
                  buf[0] += "ERROR, non-numeric instance property ["+p+"] in array:\n";
                }
                else {
                  continue; // see next: if(isArr) {
                }
              }
              ++nInst;
              isProt = false;
            }
            else {
              ++nProt;
              if(!protos) {
                continue;
              }
              isProt = true;
            }
            if(m > 0) {
              s = p + ": " +
                  (v && pT === t && v === u ? (!d ? "ref THIS\n" : "ref SELF\n") : // reference check
                  _nspct(v, protos, fb, m - 1, d + 1) ); // recursion
              if(!isProt) {
                buf[0] += (ind + s);
              }
              else {
                buf[ pT !== "function" ? 1 : 2 ] += (ind + "... " + s);
              }
            }
          }
          if(isArr) {
            nInst = u.length;
            if(m > 0) {
              for(p = 0; p < nInst; p++) {
                buf[0] += ind + p + ": " +
                    ((v = u[p]) && pT === t && v === u ? (!d ? "ref THIS\n" : "ref SELF\n") : // reference check
                    _nspct(v, protos, fb, m - 1, d + 1) ); // recursion
              }
            }
          }
        }
        catch(er) {
          em = " failed to inspect property " + p + ", type " + pT;
        }
        return "(" + t + ":" + nInst + (isArr ? "" : ("|" + nProt)) + ")" + em + (isArr ? " [" : " {") + "\n" +
            (m < 1 ? "" : (buf.join(""))) +
            new Array(d + 1).join(".  ") + (isArr ? "]" : "}") + "\n";
      }
      return "RECURSION LIMIT\n";
    };
    /**
     * Find file and line.
     *
     * @ignore
     * @private
     * @return {str}
     */
    _fL = function() {
      var ar, le, i, v, p;
      //  Find call (first trace line outside this file).
      try {
        throw new Error();
      }
      catch(er) {
        ar = _trc(er);
      }
      if(typeof ar === "string" && ar.indexOf("\n") && (le = (ar = ar.split("\n")).length)) {
        for(i = 0; i < le; i++) {
          if(i && (p = (v = ar[i]).lastIndexOf("@")) !== -1 && v.indexOf("/inspect/js/inspect") === -1) {
            return v.substr(p);
          }
        }
      }
      return "@n/a";
    };
    /**
     * Log to backend.
     *
     * @ignore
     * @private
     * @param {str} [category]
     *  - default: info
     * @param {str} [header]
     *  - gets truncated at length 255
     * @param {str} [body]
     *  - gets truncated at length 500000
     * @param {str} [severity]
     *  - default: info
     * @param {str} [type]
     *  - default: info
     *  - values: info | inspect | trace
     * @return void
     */
    _log = function(category, header, body, severity, type) {
      var h = "" + header, b = "" + body, hMax = 255, bMax = 500000;
      //  Deliberately dont tell if header or body is too long, nor if logging fails.
      $.ajax({
        url: "/inspect/ajax/" + $.cookie("inspect__sc") + ":" + (++_logNo) + "/" + (severity || "info"),
        type: "POST",
        data: {
          log_no: _logNo,
          type: type || "info",
          category: category || "info",
          header: h.substr(0, hMax),
          body: b.substr(0, bMax)
        },
        dataType: "json", // expects json formatted response data
        cache: false,
        /**
          * @return {void}
          * @param {obj} responseData
          *  - (bool) success
          *  - (string) error
          * @param {str} textStatus
          * @param {obj} jqXHR
          */
        success: function(oResp, textStatus, jqXHR) {
          /* // Uncomment for debugging.
          if(textStatus === "success" &&  _typeOf(oResp) === "object" && oResp.success && oResp.logNo) {
            //_cnsl("Log " + oResp.logNo + " succeeded");
          }
          else {
            inspect({
                textStatus: textStatus,
                response_data: oResp
            }, "inspect.logMessage() failed");
          }
          */
        },
        error: function(jqXHR, textStatus, errorThrown) {
          /* // Uncomment for debugging.
          inspect({
              textStatus: textStatus,
              errorThrown: errorThrown
          }, "inspect.logMessage() failed");
          */
        }
      });
    };
    /**
     * Basic info for functions logging to backend.
     *
     * @ignore
     * @private
     * @return {str}
     */
    _lgM = function() {
      return _fL() + "\n" +
          "url: " + _loc + "\n" +
          "browser: " + _ua + "\n";
    };

    //  Public.

    /**
     * Inspect variable and send output to console log.
     *
     * Available as inspect(), Inspect(), Drupal.inspect().
     * Documented as class, but is a function (jsdoc limitation).
     *
     * Object options flags (any number of):
     *  - (int) depth (default and absolute max: 10)
     *  - (str) message (default empty)
     *  - (bool) protos (default not: do only report number of prototypal properties of objects)
     *  - (bool) func_body (default not: do not print function body)
     *
     * Integer option:
     *  - interpreted as depth
     *
     * Boolean option:
     *  - interpreted as protos
     *
     * String options flags:
     *  - 'protos' ~ report prototypal properties of objects
     *  - 'func_body' ~ print function body
     *  - otherwise interpreted as message
     *
     * Reference checks:
     *  - only child versus immediate parent
     *  - doesnt allow to recurse deeper than 10
     *
     * Prototype properties are marked with prefix "... ", function's static members with prefix ".. ".
     * Detects if an Array property doesnt have numeric key.
     * Risky procedures are performed within try-catch.
     *
     * @name inspect
     * @constructor
     * @class
     * @singleton
     * @param {mixed} u
     * @param {obj|int|bool|str} [options]
     *  - obj: options
     *  - int: depth
     *  - bool: protos
     *  - str: message
     * @return {void}
     *  - logs to browser console, if exists
     */
    f = function(u, options) {
      var o = _rslv(options), ms = o.message || "";
      _cnsl(
        (!ms ? "" : (ms + ":\n")) + "[Inspect " + _fL() + "]\n" + _nspct(u, o.protos, o.func_body, o.depth)
      );
    };
    /**
     * Use for checking if that window.inspect is actually the one we are looking for (see example).
     * For the noactions type of this script, the value of .tcepsnI is false.
     * The name of the property is just something unlikely to exist (and the class name backwards).
     * @example
//  Check if inspect exists, and that it's the active sort.
if(typeof window.inspect === "function" && inspect.tcepsnI) {
  inspect("whatever");
}
     * @name inspect.tcepsnI
     * @type {bool}
     */
    /**
     * @name inspect.tcepsni
     * @type {bool}
     */
    f.tcepsnI = f.tcepsni = true;
    /**
     * Deprecated - use inspect(), Inspect() or Drupal.inspect() instead.
     *
     * Logs trace to console, and then calls inspect().
     * This function will be removed some day.
     *
     * @deprecated
     * @function
     * @name inspect.variable
     * @param {mixed} u
     * @param {obj|int|bool|str} [options]
     *  - see inspect()
     * @return {void}
     */
    f.variable = function(u, options) {
      //  @todo: remove this function from some version > 7.x-3.1, obsolete since that version. Remove when few use <3.1.
      try {
        throw new Error("inspect.variable is deprecated, use inspect() instead");
      }
      catch(er) {
        _cnsl(_trc(er));
      }
      f(u, options);
    };
    /**
     * Inspect variable and get output as string.
     *
     * @function
     * @name inspect.get
     * @param {mixed} u
     * @param {obj|int|bool|str} [options]
     *  - see inspect()
     * @return {str}
     */
    f.get = function(u, options) {
      var o = _rslv(options), ms = o.message || "";
      return (!ms ? "" : (ms + ":\n")) + _nspct(u, o.protos, o.func_body, o.depth);
    };
    /**
     * Deprecated - use .get() instead.
     *
     * Logs trace to console, and then calls .get().
     * This function will be removed some day.
     *
     * @deprecated
     * @function
     * @name inspect.variableGet
     * @param {mixed} u
     * @param {obj|int|bool|str} [options]
     *  - see inspect()
     * @return {str}
     */
    f.variableGet = function(u, options) {
      //  @todo: remove this function from some version > 7.x-3.1, obsolete since that version. Remove when few use <3.1.
      try {
        throw new Error("inspect.variableGet is deprecated, use inspect.get() instead");
      }
      catch(er) {
        _cnsl(_trc(er));
      }
      return f.get(u, options);
    };
    /**
     * Inspect variable and log output to backend.
     *
     * Additional option flags:
     *  - (str) category (default: inspect)
     *  - (str) severity (default: debug)
     *
     * Also logs current file and line, url, and browser plus useragent.
     *
     * @function
     * @name inspect.log
     * @param {mixed} u
     * @param {obj|int|bool|str} [options]
     *  - see inspect()
     * @return {void}
     */
    f.log = function(u, options) {
      var o = _rslv(options), sv;
      _log(
        o.category || "inspect",
        o.message || "",
        "[frontend Inspect - depth:" + (o.depth || 10) + "]" + "\n" +
            _lgM() +
            _nspct(u, o.protos, o.func_body, o.depth),
        (sv = o.severity) !== undefined ? sv : "debug",
        'inspect' // Type.
      );
    };
    /**
     * All native types are reported in lowercase (like native typeof does).
     * If given no arguments: returns "inspect".
     * Types are:
     * - native, typeof: object string number
     * - native, corrected: function array date regexp image
     * - window, document, document.documentElement (not lowercase)
     * - element, checked via .getAttributeNode
     * - text node: textNode
     * - attribute node: attributeNode
     * - event: event (native and prototyped W3C Event and native IE event)
     * - jquery
     * - emptyish and bad: undefined, null, NaN, infinite
     * - custom or prototyped native: all classes having a typeOf() method.
     * RegExp is an object of type regexp (not a function - gecko/webkit/chromium).
     * Does not check if Date object is NaN.
     *
     * @function
     * @name inspect.typeOf
     * @param {mixed} u
     * @return {str}
     */
    f.typeOf = _typeOf;
    /**
     * Trace error and send output to console log.
     *
     * @example
try {
  throw new Error("¿Que pasa?");
}
catch(er) {
  inspect.trace(er, "Class.method()");
}
     * @function
     * @name inspect.trace
     * @param {Error} er
     * @param {str} [ms]
     *  - optional message to prepend
     * @return {void}
     *  - logs to browser console, if exists
     */
    f.trace = function(er, ms) {
      _cnsl(
        (!ms ? "" : (ms + ":\n")) + "[Inspect trace " + _fL() + "]\n" + _trc(er)
      );
    };
    /**
     * Trace error and get output as string.
     *
     * @function
     * @name inspect.traceGet
     * @param {Error} er
     * @param {str} [ms]
     *  - optional message to prepend
     * @return {void}
     */
    f.traceGet = function(er, ms) {
      return (!ms ? "" : (ms + ":\n")) + _trc(er);
    };
    /**
     * Trace error and log trace to backend.
     *
     * Object options flags (any number of):
     *  - (str) message (default empty)
     *  - (str) category (default: inspect trace)
     *  - (str) severity (default: debug)
     *
     * String options flags:
     *  - interpreted as message
     *
     * Also logs current file and line, url, and browser plus useragent.
     *
     * @function
     * @name inspect.traceLog
     * @param {Error} er
     * @param {obj|str} [options]
     *  - obj: options
     *  - str: message
     * @return {void}
     */
    f.traceLog = function(er, options) {
      var o = _rslv(options), ms, sv, ua;
      _log(
        o.category || "inspect trace",
        ms = o.message || "",
        "[frontend Inspect trace]\n" +
            _lgM() +
            _trc(er),
        (sv = o.severity) !== undefined ? sv : "debug",
        'trace' // Type.
      );
    };
    /**
     * Analyze function arguments, or an options object.
     *
     * @function
     * @name inspect.argsGet
     * @param {obj|collection|arr} args
     * @param {arr} [names]
     *  - optional array of argument names
     * @return {str}
     */
    f.argsGet = function(args, names) {
      var a = args, t = _typeOf(a), le, keys, nKs, s, i;
      if(!a) {
        return "falsy arg args";
      }
      if(t === "array") {
        if(!(le = a.length)) {
          return "0: none";
        }
        nKs = !(keys = names) ? 0 : keys.length; // use arg names as keys
      }
      else if(!(le = ((keys = _oKeys(a)).length))) { // arguments or object having no length
        return "0: none";
      }
      else if(keys[0] === "0") { // arguments, arrayish
        t = "array";
        nKs = !(keys = names) ? 0 : keys.length; // use arg names as keys
      }
      else { // object
        nKs = le;
        s = ""+le+":";
      }
      s = ""+le+":";
      for(i = 0; i < le; i++) {
        s += (i ? ", " : " ") + "#" + i + (nKs <= i ? "" : ("("+keys[i]+")")) + " " +
            _nspct(t === "array" ? a[i] : a[ keys[i] ], 0, 0, 0).
            replace(/\n/, ""); // get rid of trailing newline
      }
      return s;
    };
    /**
     * Inspect event listeners added via jQuery, and send output to console.
     *
     * List events of all elements matched by the jQuery selector, as array of objects containing an element and a listener list.
     * Defaults to display bodies of functions (options func_body).
     *
     * @function
     * @name inspect.events
     * @param {string|obj} selector
     *  - jQuery selector
     * @param {obj|int|bool|str} [options]
     *  - see inspect()
     * @return {void}
     *  - logs to browser console, if exists
     */
    f.events = function(selector, options) {
      var o = _rslv(options), ms = o.message || "";
      o.message = "";
      _cnsl(
        (!ms ? "" : (ms + ":\n")) + "[Inspect events " + _fL() + "]\n" + f.eventsGet(selector, null, null, o)
      );
    };
    /**
     * Inspect event listeners added via jQuery, and get output as string.
     *
     * List events of all elements matched by the jQuery selector, as array of objects containing an element and a listener list.
     * Defaults to display bodies of functions (options func_body).
     *
     * @function
     * @name inspect.eventsGet
     * @param {string|obj} selector
     *  - jQuery selector
     * @param {obj|int|bool|str} [options]
     *  - see inspect()
     * @return {str}
     */
    f.eventsGet = function(selector, options) { // hidden 4th argument: resolved options
      var o = arguments[3] || _rslv(options), ms = o.message || "", fb = o.func_body, s, u = selector, jq = $(u), d, a = [];
      if(!jq.get(0)) {
        s = "arg selector, type " + _typeOf(u) + ", value [" + u + "], matches no element(s)";
      }
      else {
        if(fb === undefined) {
          fb = true;
        }
        //  No support for jQuery events listed under 'jquery[long hex number]' bucket (observed via judy); cant replicate that pattern.
        jq.each(function() { // Using jQuery.each() to avoid reset values during iteration.
          a.push({
            element: this,
            listeners: (d = $(this).data()) && d.events && d.hasOwnProperty("events") ? d.events : null
          });
        });
        s = "selector: " + u + "\n" + _nspct(a, o.protos, fb, o.depth)
      }
      return (!ms ? "" : (ms + ":\n")) + s;
    };
    /**
     * Inspect event listeners added via jQuery, and log output to backend.
     *
     * List events of all elements matched by the jQuery selector, as array of objects containing an element and a listener list.
     * Defaults to display bodies of functions (options func_body).
     *
     * Additional option flags:
     *  - (str) category (default: inspect)
     *  - (str) severity (default: debug)
     *
     * String options flags:
     *  - interpreted as message
     *
     * Also logs current file and line, url, and browser plus useragent.
     *
     * @function
     * @name inspect.eventsLog
     * @param {string|obj} selector
     *  - jQuery selector
     * @param {obj|int|bool|str} [options]
     *  - see inspect()
     * @return {void}
     */
    f.eventsLog = function(selector, options) {
      var o = _rslv(options), sv, ms = o.message || "";
      o.message = "";
      _log(
        o.category || "inspect",
        ms,
        "[frontend Inspect events]\n" +
            _lgM() +
            f.eventsGet(selector, null, null, o),
        (sv = o.severity) !== undefined ? sv : "debug",
        'inspect' // Type.
      );
    };
    /**
     * Send message to browser console, if exists.
     *
     * @function
     * @name inspect.console
     * @param {mixed} [ms]
     * @return {void}
     */
    f.console = _cnsl;
    /**
     * Log message to backend.
     *
     * @function
     * @name inspect.logMessage
     * @param {str} [category]
     *  - default: info
     * @param {str} [header]
     *  - gets truncated at length 255
     * @param {str} [body]
     *  - gets truncated at length 500000
     * @param {str} [severity]
     *  - default: info
     * @param {str} [type]
     *  - default: info
     *  - values: info | inspect | trace
     * @return void
     */
    f.logMessage = _log;
    /**
      * Toggle display of all inspection outputs on current page.
      *
      * This function is always implemented, even for no-action no-fold.
      *
      * @function
      * @name inspect.toggleOutput
      * @return {void}
      */
    f.toggleDisplay = function() {
      var u;
      if(_shw > 0) {
        $("." + _cls).hide();
      }
      else if(_fld < 0) {
        $("pre."+ _cls).show();
      }
      else {
        $("pre."+ _cls).each(function(){
          $((u = $(this).prev("div."+ _cls).get(0)) ? u : this).show();
        });
      }
      _shw *= -1;
    };
    /**
      * Toggle inspection folding, unless folding is disabled.
      *
      * Folding is not supported for IE<9.
      *
      * @function
      * @name inspect.toggleFolding
      * @return {void}
      */
    f.toggleFolding = function() {
      var u;
      if(!(u = Drupal.settings.inspect) || !u.foldingOff) {
        if(_fld > 0) {
          $("div." + _cls).hide();
          $("pre." + _cls).show();
        }
        else {
          $("pre." + _cls).hide();
          $("div." + _cls).show();
        }
        _fld *= -1;
        _shw = 1;
      }
    };

    (function() {
      var u = window.navigator, ua = "no window.navigator.userAgent", x, y, nm, v = "?", b = {}, s;
      if(u && typeof u === "object" && (u = u.userAgent) && typeof u === "string") {
        if((s = (ua = u).toLowerCase()).indexOf(x = "msie") > 0) {
          nm = x;
        }
        else if(s.indexOf(x = "chrome") > 0 || s.indexOf(x = "safari") > 0) {
          nm = x;
          b.webkit = (y = /^.+webkit\/(\d+\.\d+)[^\d].+$/).test(s) ? parseFloat(s.replace(y, "$1")) : true;
        }
        else if(s.indexOf(x = "firefox") > 0) {
          nm = x;
          b.mozilla = true;
        }
        else if(s.indexOf(x = "opera") > 0) {
          nm = x;
        }
        if(nm) {
          b[nm] = (x = new RegExp("^.+" + nm + ".(\\d+\\.\\d+)([^\\d].+)?$")).test(s) ? parseFloat(v = s.replace(x, "$1")) : true;
          b.version = v;
        }
        else {
          nm = "Unknown";
        }
      }
      _ua = nm + " " + v + " [" + ua + "]";
      /**
       * Lists browser version (as float) by useragent identifier.
       *
       * Supported browsers/identifiers:
       *  - msie ~ Internet Explorer
       *  - firefox
       *  - mozilla ~ Firefox, but version is simply true (not float)
       *  - chrome ~ Google Chrome
       *  - safari
       *  - webkit ~ Google Chrome or Apple Safari, version is (Apple)Webkit version as float
       *  - opera
       *  - version ~ version number as string (for Firefox: the Firefox version, not the Gecko version).
       *
       * @example
//  Check if browser is Internet Explorer < 8
var ie;
if((ie = inspect.browser.msie) && ie < 8) {
  inspect.console("Get a decent browser, bro'");
}
       * @name inspect.browser
       * @type {obj}
       */
      f.browser = b;
    })();

    return f;
  })();
})(jQuery);